@@ -167,6 +167,9 @@ int64_t ProcessReads(MasterProcessor& MP, const  ProgramOptions& opt) {
 
   // for each file
   std::cerr << "[quant] finding pseudoalignments for the reads ..."; std::cerr.flush();
+  if (opt.verbose) {
+    std::cerr << std::endl;
+  }
 
   /*if (opt.pseudobam) {
     bam_hdr_t *t = createPseudoBamHeader(index);
@@ -177,7 +180,11 @@ int64_t ProcessReads(MasterProcessor& MP, const  ProgramOptions& opt) {
   MP.processReads();
   numreads = MP.numreads;
   nummapped = MP.nummapped;
-  std::cerr << " done" << std::endl;
+  if (opt.verbose) {
+    std::cerr << std::endl << "[quant] done " << std::endl;
+  } else {
+    std::cerr << " done" << std::endl;
+  }
 
   //std::cout << "betterCount = " << betterCount << ", out of betterCand = " << betterCand << std::endl;
 
@@ -237,16 +244,70 @@ int64_t ProcessBUSReads(MasterProcessor& MP, const  ProgramOptions& opt) {
   // for each file
   std::cerr << "[quant] finding pseudoalignments for the reads ..."; std::cerr.flush();
 
-  /*if (opt.pseudobam) {
-    bam_hdr_t *t = createPseudoBamHeader(index);
-    index.writePseudoBamHeader(std::cout);
-  }*/
+  if (opt.genomebam) {
+    /*
+    // open bam files for writing
+    MP.bamh = createPseudoBamHeaderGenome(MP.model);
+    MP.bamfps = new htsFile*[MP.numSortFiles];
+    for (int i = 0; i < MP.numSortFiles; i++) {
+      MP.bamfps[i] = sam_open((opt.output + "/tmp." + std::to_string(i) + ".bam").c_str(), "wb1");
+      int r = sam_hdr_write(MP.bamfps[i], MP.bamh);
+    }
+
+    // assign breakpoints to chromosomes
+    MP.breakpoints.clear();
+    MP.breakpoints.assign(MP.numSortFiles -1 , (((uint64_t)-1)<<32));
+    std::vector<std::vector<std::pair<uint32_t, uint32_t>>> chrWeights(MP.model.chr.size());
+    for (const auto& t : MP.model.genes) {
+      if (t.chr != -1 && t.stop > 0) {
+        chrWeights[t.chr].push_back({t.stop, 1});
+      }      
+    }
+
+    double sum = 0;
+    for (const auto &chrw : chrWeights) {
+      for (const auto &p : chrw) {
+        sum += p.second;
+      }
+    }
+    double bpLimit = sum / (MP.numSortFiles-1);
+
+    for (auto& chrw : chrWeights) {
+      // sort each by stop point
+      std::sort(chrw.begin(), chrw.end(), [](std::pair<uint32_t, uint32_t> a, std::pair<uint32_t, uint32_t> b) { return a.first < b.first;});
+    }
+
+    double bp = 0.0;
+    int ifile = 0;
+    for (int i = 0; i < chrWeights.size(); i++) {
+      const auto &chrw = chrWeights[i];
+      for (const auto &x : chrw) {
+        bp += x.second;
+        if (bp > bpLimit) {
+          uint64_t pos = ((uint64_t) i) << 32 | ((uint64_t) x.first+1) << 1;
+          MP.breakpoints[ifile] = pos;
+          ifile++;
+          while (bp > bpLimit) {
+            bp -= bpLimit;
+          }
+        }
+      }
+    }
+    */
+  }
+
+
+
 
   
   MP.processReads();
   numreads = MP.numreads;
   nummapped = MP.nummapped;
-  std::cerr << " done" << std::endl;
+  if (opt.verbose) {
+    std::cerr << std::endl << "[quant] done " << std::endl;
+  } else {
+    std::cerr << " done" << std::endl;
+  }
 
   //std::cout << "betterCount = " << betterCount << ", out of betterCand = " << betterCand << std::endl;
 
@@ -865,6 +926,7 @@ ReadProcessor::ReadProcessor(const KmerIndex& index, const ProgramOptions& opt,
    if (opt.batch_mode) {
      assert(id != -1);
      batchSR.files = opt.batch_files[id];
+     batchSR.nfiles = opt.batch_files[id].size();
      batchSR.reserveNfiles(opt.batch_files[id].size());
      if (opt.umi) {
        batchSR.umi_files = {opt.umi_files[id]};
@@ -951,6 +1013,7 @@ void ReadProcessor::processBuffer() {
   std::vector<int> vtmp;
   std::vector<int> u;
 
+
   u.reserve(1000);
   v1.reserve(1000);
   v2.reserve(1000);
@@ -1199,13 +1262,17 @@ void ReadProcessor::processBuffer() {
       */
     }
     
+    if (mp.opt.verbose && numreads > 0 && numreads % 1000000 == 0 ) {   
+      int nmap = mp.nummapped;
+      for (int i = 0; i < counts.size(); i++) {
+        nmap += counts[i];
+      }
+      nmap += newEcs.size();
 
-
-
-    /*
-    if (opt.verbose && numreads % 100000 == 0 ) {
-      std::cerr << "[quant] Processed " << pretty_num(numreads) << std::endl;
-    }*/
+      std::cerr << '\r' << (numreads/1000000) << "M reads processed (" 
+        << std::fixed << std::setw( 3 ) << std::setprecision( 1 ) << ((100.0*nmap)/double(numreads))
+        << "% pseudoaligned)"; std::cerr.flush();
+    }
   }
 
 }
@@ -1223,7 +1290,7 @@ void ReadProcessor::clear() {
 
 
 BUSProcessor::BUSProcessor(const KmerIndex& index, const ProgramOptions& opt, const MinCollector& tc, MasterProcessor& mp, int _id, int _local_id) :
- paired(!opt.single_end), bam(opt.bam), num(opt.num), tc(tc), index(index), mp(mp), id(_id), local_id(_local_id) {
+ paired(!opt.single_end), bam(opt.bam), num(opt.num), tc(tc), index(index), mp(mp), id(_id), local_id(_local_id), numreads(0) {
    // initialize buffer
    bufsize = mp.bufsize;
    buffer = new char[bufsize];  
@@ -1288,17 +1355,16 @@ void BUSProcessor::operator()() {
       // release the reader lock
     }
     // do the same for BUS ?!?
-    /*    
+    
     pseudobatch.aln.clear();
-    pseudobatch.batch_id = readbatch_id;*/
+    pseudobatch.batch_id = readbatch_id;
     // process our sequences
     processBuffer();
 
     // update the results, MP acquires the lock
     std::vector<std::pair<int, std::string>> ec_umi;
     std::vector<std::pair<std::vector<int>, std::string>> new_ec_umi;
-    PseudoAlignmentBatch pseudobatch;
-    mp.update(counts, newEcs, ec_umi, new_ec_umi, paired ? seqs.size()/2 : seqs.size(), flens, bias5, pseudobatch, bv, newB, &bc_len[0], &umi_len[0], id, local_id);
+    mp.update(counts, newEcs, ec_umi, new_ec_umi, seqs.size() / mp.opt.busOptions.nfiles , flens, bias5, pseudobatch, bv, newB, &bc_len[0], &umi_len[0], id, local_id);
     clear();
   }
 }
@@ -1316,28 +1382,35 @@ void BUSProcessor::processBuffer() {
   memset(&bc_len[0], 0, 33);
   memset(&umi_len[0], 0, 33);
 
-  const char* s1 = 0;
-  const char* s2 = 0;
-  const char* s3 = 0;
-  int l1=0,l2=0,l3=0;
-
-  const char *s[3] = {0,0,0};
-  int l[3] = {0,0,0};
+  const BUSOptions& busopt = mp.opt.busOptions;
+ 
   char buffer[100];
   memset(buffer, 0, 100);
   char *umi = &(buffer[50]);
   char *bc  = &(buffer[0]);
+  //char *seqbuffer[1000];
+  std::string seqbuffer;
+  seqbuffer.reserve(1000);
+  
+  // actually process the sequence
   
-  // actually process the sequences
-  const BUSOptions& busopt = mp.opt.busOptions;
   int incf, jmax;
   if (bam) {
     incf = 1;
     jmax = 2;
+
   } else {
     incf = busopt.nfiles-1;
     jmax = busopt.nfiles;
   }
+
+  std::vector<const char*> s(jmax, nullptr);
+  std::vector<int> l(jmax,0);
+
+  bool singleSeq = busopt.seq.size() ==1 ;
+  const BUSOptionSubstr seqopt = busopt.seq.front();
+
+
   //int incf = (bam) ? 1 : busopt.nfiles-1;
   for (int i = 0; i + incf < seqs.size(); i++) {
     for (int j = 0; j < jmax /*(bam) ? 2 : busopt.nfiles*/; j++) {
@@ -1347,8 +1420,24 @@ void BUSProcessor::processBuffer() {
     i += incf;
     
     // find where the sequence is
-    const char *seq = s[busopt.seq.fileno] + busopt.seq.start;
-    int seqlen = l[busopt.seq.fileno];
+    const char *seq = nullptr;
+    
+    size_t seqlen = 0;
+    if (singleSeq) {
+      seq = s[seqopt.fileno] + seqopt.start;
+      seqlen = l[seqopt.fileno];
+    } else {
+      seqbuffer.clear();
+      for (int j = 0; j < busopt.seq.size(); j++) {
+        const auto &sopt = busopt.seq[j];
+        int cplen = (sopt.stop == 0) ? l[sopt.fileno]-sopt.start : sopt.stop - sopt.start;
+        seqbuffer.append(s[sopt.fileno] + sopt.start, cplen);
+        seqbuffer.push_back('N');        
+      }
+      seqbuffer.push_back(0);
+      seq = seqbuffer.c_str();
+      seqlen = seqbuffer.size();
+    }
 
     // copy the umi
     int umilen = (busopt.umi.start == busopt.umi.stop) ? l[busopt.umi.fileno] - busopt.umi.start : busopt.umi.stop - busopt.umi.start;
@@ -1430,8 +1519,43 @@ void BUSProcessor::processBuffer() {
         // push back BUS record
         b.ec = ec;
         bv.push_back(b);
-      }
+      }      
     }
+
+    if (mp.opt.pseudobam) {
+      PseudoAlignmentInfo info;
+      info.id = i;
+      info.paired = false;
+      uint32_t f = 0;
+      info.barcode = stringToBinary(bc, blen, f);
+      info.UMI = stringToBinary(umi, umilen, f);
+      if (!u.empty()) {
+        info.r1empty = v.empty();
+        KmerEntry val;
+        info.k1pos = findFirstMappingKmer(v,val);
+        info.k2pos = -1;
+                
+        if (ec != -1) {
+          info.ec_id = ec;
+        } else {
+          info.ec_id = -1;
+          info.u = u; // copy
+        }
+      }
+      pseudobatch.aln.push_back(std::move(info));
+    }
+
+    if (mp.opt.verbose && numreads > 0 && numreads % 1000000 == 0 ) {   
+        int nmap = mp.nummapped;
+        for (int i = 0; i < counts.size(); i++) {
+          nmap += counts[i];
+        }
+        nmap += newEcs.size();
+
+        std::cerr << '\r' << (numreads/1000000) << "M reads processed (" 
+          << std::fixed << std::setw( 3 ) << std::setprecision( 1 ) << ((100.0*nmap)/double(numreads))
+          << "% pseudoaligned)"; std::cerr.flush();
+      }
   }
 }
 
@@ -1458,6 +1582,7 @@ AlnProcessor::AlnProcessor(const KmerIndex& index, const ProgramOptions& opt, Ma
      /* need to check this later */
      assert(id != -1);
      batchSR.files = opt.batch_files[id];
+     batchSR.nfiles = opt.batch_files[id].size();
      batchSR.reserveNfiles(opt.batch_files[id].size());
      if (opt.umi) {
        batchSR.umi_files = {opt.umi_files[id]};
@@ -1976,26 +2101,56 @@ void AlnProcessor::processBufferGenome() {
   std::vector<std::pair<int,double>> ua;
   u.reserve(1000);
   ua.reserve(1000);
+
+  int bclen = 0;
+  int umilen = 0;
   
   int genome_auxlen = default_genome_auxlen;
   if (!useEM) {
     genome_auxlen -= 7;
   }
+  if (mp.opt.bus_mode) {
+    //todo replace with what is written to busfile
+    bclen = mp.opt.busOptions.getBCLength();
+    if (bclen == 0) {
+      bclen = 32;
+    }
+    umilen = mp.opt.busOptions.getUMILength();
+    if (umilen == 0) {
+      umilen = 32;
+    }
+
+    genome_auxlen += (bclen + umilen) + 8;
+  }
+  std::string bc, umi;
   
 
   Kmer km1,km2;
   KmerEntry val1, val2;
+  if  (mp.opt.bus_mode) {
+    paired = false;
+  }
+  bool readpaired = paired || (mp.opt.bus_mode && mp.opt.busOptions.nfiles == 2);
+  
+  
 
   for (int i = 0; i < n; i++) {
     bam1_t b1,b2, b1c, b2c;
-    int si1 = (paired) ? 2*i : i;
-    int si2 = (paired) ? 2*i +1 : -1;
+    int si1 = (readpaired) ? 2*i : i;
+    int si2 = (readpaired) ? 2*i +1 : -1;
+    
     int rlen1 = seqs[si1].second;
     int rlen2;
-    if (paired) {
+    if (readpaired) {
       rlen2 = seqs[si2].second;
     }
     
+    if (mp.opt.busOptions.seq.front().fileno == 1) {
+      std::swap(si1,si2);
+      std::swap(rlen1,rlen2);
+    }
+    
+
     // fill in the bam core info
     b1.core.tid = -1;
     b1.core.pos = -1;
@@ -2029,6 +2184,22 @@ void AlnProcessor::processBufferGenome() {
       fillBamRecord(b2, nullptr, seqs[si2].first, names[si2].first,  quals[si2].first, seqs[si2].second, names[si2].second, pi.r2empty, genome_auxlen);
       // b2.id = idnum++;
     }
+    
+    if (mp.opt.bus_mode) {
+      b1.data[b1.l_data] = 'C';
+      b1.data[b1.l_data+1] = 'R';
+      b1.data[b1.l_data+2] = 'Z';
+      bc = binaryToString(pi.barcode, bclen);
+      memcpy(b1.data + b1.l_data + 3, bc.c_str(), bclen+1);
+      b1.l_data += bclen + 4;
+      
+      b1.data[b1.l_data] = 'U';
+      b1.data[b1.l_data+1] = 'R';
+      b1.data[b1.l_data+2] = 'Z';
+      umi = binaryToString(pi.UMI, umilen);
+      memcpy(b1.data + b1.l_data + 3, umi.c_str(), umilen+2);
+      b1.l_data += umilen + 4;
+    }
 
     if (pi.r1empty && pi.r2empty) {
       bv.push_back(b1);
@@ -2111,7 +2282,7 @@ void AlnProcessor::processBufferGenome() {
           
         // set aux
         float zero = 0.0;
-        assert(b1.l_data + genome_auxlen <= b1.m_data);
+        //assert(b1.l_data + genome_auxlen <= b1.m_data);
         
         if (useEM) {
           b1.data[b1.l_data] = 'Z';
@@ -2121,7 +2292,7 @@ void AlnProcessor::processBufferGenome() {
           b1.l_data += 7;
 
           if (paired) {
-            assert(b2.l_data + genome_auxlen <= b2.m_data);
+            //assert(b2.l_data + genome_auxlen <= b2.m_data);
             b2.data[b2.l_data] = 'Z';
             b2.data[b2.l_data+1] = 'W';
             b2.data[b2.l_data+2] = 'f';
@@ -2165,7 +2336,6 @@ void AlnProcessor::processBufferGenome() {
         if (!pi.r1empty) {
           km1 = Kmer(seqs[si1].first + pi.k1pos);
           strInfo1 = strandednessInfo(km1, val1, ua);
-
         }
         if (paired && !pi.r2empty) {
           km2 = Kmer(seqs[si2].first + pi.k2pos);
@@ -2743,7 +2913,8 @@ bool FastqSequenceReader::fetchSequences(char *buf, const int limit, std::vector
           umis.emplace_back(std::move(umi));
         }
 
-        flags.push_back(numreads++);
+        numreads++;
+        flags.push_back(numreads);
       } else {
         return true; // read it next time
       }
@@ -2759,7 +2930,7 @@ bool FastqSequenceReader::fetchSequences(char *buf, const int limit, std::vector
 }
 
 // move constructor
-#if 1
+
 FastqSequenceReader::FastqSequenceReader(FastqSequenceReader&& o) :
   nfiles(o.nfiles),
   numreads(o.numreads),
@@ -2770,34 +2941,16 @@ FastqSequenceReader::FastqSequenceReader(FastqSequenceReader&& o) :
   files(std::move(o.files)),
   umi_files(std::move(o.umi_files)),
   f_umi(std::move(o.f_umi)),
-  current_file(o.current_file) {
+  current_file(o.current_file),
+  seq(std::move(o.seq)) {
 
   o.fp.resize(nfiles);
   o.l.resize(nfiles, 0);
   o.nl.resize(nfiles, 0);
-  o.state = false;
-}
-#endif
-#if 0
-FastqSequenceReader::FastqSequenceReader(FastqSequenceReader&& o) {
-  fp          = (std::move(o.fp));
-  l           = (std::move(o.l));
-  nl          = (std::move(o.nl));
-  paired      = (o.paired);
-  nfiles      = (o.nfiles);
-  files       = (std::move(o.files)),
-  umi_files   = (std::move(o.umi_files));
-  f_umi       = (std::move(o.f_umi));
-  current_file= (o.current_file);
-  seq         = (std::move(o.seq));
-
-  o.fp.resize(nfiles);
-  o.l.resize(nfiles, 0);
-  o.nl.resize(nfiles, 0);
-  o.state = false;
   o.seq.resize(nfiles, nullptr);
+  o.state = false;
 }
-#endif
+
 const std::string BamSequenceReader::seq_enc = "=ACMGRSVTWYHKDBN";
 
 BamSequenceReader::~BamSequenceReader() {
